#!/usr/bin/perl -w
use Storable; # For caching

# Some settings.  At some point I should figure out a better system
# for these... they are legacy from Notes2.
%properties = (Debug => 'False', Bibliography => 'xxx');

# This file converts pmids, dois, and isbns to citations.  It is
# lifted directly from Notes2.

if ($#ARGV != 0) {
    die "\nUsage: ./latexify-notes.pl [CITESTRING]\nCITESTRING is in the format [type]:[id], where [type] is 'doi', 'isbn', or 'pmid', and [id] is the source's doi, isbn, or pmid number.  Outputs the parenthetical text, followed by a tab, and then the link.";
}

sub get_citation_from_doi {
    my $doi = shift;
    my $output = "";
    # Resolve the DOI and write it to a text file
    $command = "curl -LH \"Accept: application/x-research-info-systems\" 'http://dx.doi.org/$doi'";
    $bibcommand = "curl -LH \"Accept: text/bibliography; style=bibtex\" 'http://dx.doi.org/$doi'";
    if ($properties{'Debug'} ne "True") {
        $output = command_or_cache($command);
        if (exists $properties{'Bibliography'} && $properties{'Bibliography'} eq "auto") {
            my $bibtex_text = command_or_cache($bibcommand) . "\n";
            my $key = $1 if ($bibtex_text =~ /\@article{([-A-Za-z0-9_]+),/);
            $key =~ s/_/-/;
            $bibtex_entries .= $bibtex_text;
            return "\\citep{$key}";
        }
    }
    else {
        $output = `false`; # Used for return code only
    }
    if ($? == 0) { # Fetch successful, exit status 0
        my ($authorlast) = $output =~ /AU\s*-\s*([^,]+),/;
        my ($year) = $output =~ /PY\s*-\s*([0-9]{4})/;
        if (defined($authorlast) && defined($year)) {
            return "($authorlast, $year)";
        }
    }
    return "(doi:$doi)";
}
# TODO get Bibtex entry for ISBNs
sub get_citation_from_isbn {
    my $isbn = shift;
    my $output = "";
    $command = "curl -L 'http://xisbn.worldcat.org/webservices/xid/isbn/$isbn?method=getMetadata&format=xml&fl=*'";
    if ($properties{'Debug'} ne "True") {
        $output = command_or_cache($command);
        # Put bibliography stuff in here when you write it
    }
    else {
        $output = `false`;
    }
    if ($? == 0) { # Fetch successful, exit status 0
        my $author = "";
        my $year = "";
        $author = $1 if ($output =~ /author="([^"]+)"/);
        $year = $1 if ($output =~ /year="([^"]+)"/);
        if ($author eq "") {
            return "($year)";
        }
        return "($author, $year)";
    }
    else {
        return "(ISBN $isbn)";
    }
}
sub make_bibtex_entry {
    my ($firstauthor, $year, $title, $jvol, $jissue, $jname, $jshortname, $jissn, $pages, $authorslist) = @_;
    $pages =~ s/-/--/;
    my $bibtex = <<END;
\@article{$firstauthor$year,
	title = {$title},
	volume = {$jvol},
	issn = {$jissn},
	number = {$jissue},
	journal = {$jshortname},
	author = {$authorslist},
	year = {$year},
	pages = {$pages}
}
END
    return $bibtex;
}
sub get_citation_from_pmid {
    my $pmid = shift;
    my $output = "";
    if ($properties{'Debug'} ne "True") {
        # First get the special ID their API needs
        my $url = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/epost.fcgi?db=pubmed&id=$pmid";
        my $command = "curl -L '$url'";
        my $web = "";
        my $key = "";
        $output = command_or_cache($command);
        if ($? != 0) { # Make sure fetch was successful
            return "(PMID: $pmid)";
        }
        $web = $1 if ($output =~ /<WebEnv>(\S+)<\/WebEnv>/);
        $key = $1 if ($output =~ /<QueryKey>(\d+)<\/QueryKey>/);
        # Now actually look it up
        $url = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?db=pubmed&query_key=$key&WebEnv=$web&rettype=xml";
        $command = "curl -L '$url'";
        $output = command_or_cache($command);
        # Use XML parser to decode data
        use XML::Simple;
        my $xml = new XML::Simple (KeyAttr=>[], ForceArray=>1);
        my $parsedbib = $xml->XMLin($output);
        my $article = $parsedbib->{PubmedArticle}->[0]->{MedlineCitation}->[0]->{Article}->[0];
        my $firstauthor = $article->{AuthorList}->[0]->{Author}->[0]->{LastName}->[0];
        $firstauthor = ucfirst(lc($firstauthor)); # Fix some faulty citations for internal use, Bibtex takes care of the rest
        my $year = $article->{Journal}->[0]->{JournalIssue}->[0]->{PubDate}->[0]->{Year}->[0];
        my $title = $article->{ArticleTitle}->[0];
        my $jvol = $article->{Journal}->[0]->{JournalIssue}->[0]->{Volume}->[0] // ''; # Defined-or operator
        my $jissue = $article->{Journal}->[0]->{JournalIssue}->[0]->{Issue}->[0] // '';
        my $jname = $article->{Journal}->[0]->{Title}->[0];
        my $jshortname = $article->{Journal}->[0]->{ISOAbbreviation}->[0];
        my $jissn = $article->{Journal}->[0]->{ISSN}->[0]->{content} // '';
        my $pages = $article->{Pagination}->[0]->{MedlinePgn}->[0] // '';
        if (ref($pages) eq 'HASH') {$pages = '';} # For when pages not specified (since it is a property, not an element)
        my @authors = ();
        foreach $author (@{$article->{AuthorList}->[0]->{Author}}) {
            push(@authors, "$author->{LastName}->[0], $author->{ForeName}->[0]");
        }
        my $authorslist = join(' and ', @authors);
        $authorslist = lc($authorslist);
        $authorslist =~ s/([\w']+)/\u\L$1/g; # Lowercase everything except first letters of words, fixes bad pubmed db entries
        if (exists $properties{'Bibliography'} && $properties{'Bibliography'} eq "auto") {
            $bibtex_entries .= make_bibtex_entry($firstauthor, $year, $title, $jvol, $jissue, $jname, $jshortname, $jissn, $pages, $authorslist) . "\n";
            return "\\citep{$firstauthor$year}";
        }

        return "($firstauthor, $year)";
    }
    else {
        return "(PMID: $pmid)";
    }
}

# This function is used by the previous three to cache URLs.  It checks
# to see if a URL has been previously retrieved, and if so, returns the
# contents from the cache.  If not, it downloads the URL, and then saves
# it to the cache and returns it.  This should not be used if content is
# not expected to be static.

$urlcachefilename = 'cache.urlcache';
%urlcache = ();
if (-e $urlcachefilename) {
    %urlcache = %{retrieve($urlcachefilename)};
}
sub command_or_cache {
    my $command = shift;
    #use Data::Dumper::Simple;
    #print Dumper $urlcache;
    #print $command;
    #print $urlcache{$command};
    if (exists $urlcache{$command}) {
        return $urlcache{$command};
    }
    else {
        $output = `$command`;
        $urlcache{$command} = $output;
        return $output;
    }
}

$citestring = $ARGV[0];
$citestring =~ m/(doi|pmid|isbn):(.+)/ or die("Error, string did not match");
if ($1 eq 'doi') {
    my $doi = $2;
    my $citetext = get_citation_from_doi($doi);
    print $citetext . "\t" . "http://dx.doi.org/" . $doi;
}
elsif ($1 eq 'isbn') {
    my $isbn = $2;
    my $citetext = get_citation_from_isbn($isbn);
    print $citetext . "\t" . "http://www.worldcat.org/isbn/" . $isbn;
}
elsif ($1 eq 'pmid') {
    my $pmid = $2;
    my $citetext = get_citation_from_pmid($pmid);
    print $citetext . "\t" . "http://www.ncbi.nlm.nih.gov/pubmed/" . $pmid;
}

# Write the cache, after having updated it from potential successful
# http requests.
store \%urlcache, $urlcachefilename;
