#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import ply.lex as lex
import ply.yacc as yacc
import re
import sys # For errors

# Exception used for grammar errors
class MathGrammarError(Exception):
    pass

# Lists of symbols
# ================

combining_chars = {'⃗': 'vec', '̃': 'tilde', '̂': 'hat', '̄': 'bar', '̉':
                   'overset{?}', '̇': 'dot', '⃝': 'circled', '̅': 'bar'}

funs = ['sin', 'cos', 'tan', 'arcsin', 'arccos', 'arctan', 'asin',
        'acos', 'atan', 'sec', 'csc', 'cot', 'ln', 'log', 'max',
        'min', 'sup', 'inf', 'ker', 'coker', 'rng', 'corng', 'var',
        'rank', 'rk', 'exp', 'erf', 'det', 'tr', 'lim', 'Cov',
        'Var', 'Bias', 'Median']

symbols = {'\+': '+',
          '\*': '*', 
           '×': '\\times', 
           '/': '/', 
           '>': '>', 
           '<': '<', 
           '<<': '\\ll', 
           '>>': '\\gg', 
           '=': '=', 
           '±': '\\pm', 
           '≤': '\\leq', 
           '≥': '\\geq', 
           '÷': '\\div', 
           '·': '\\cdot', 
           '∘': '\\circ',
           '≠': '\\neq', 
           '≠': '\\neq', 
           '≈': '\\approx', 
           '⊃': '\\supset', 
           '⊂': '\\subset', 
           '⊇': '\\supseteq', 
           '⊆': '\\subseteq', 
           '∖': '\\setminus', 
           '≡': '\\equiv', 
           '∧': '\\land', 
           '∨': '\\lor', 
           '∈': '\\in', 
           '∋': '\\ni',
           '∩': '\\cap',
           '∪': '\\cup',
           ':': ':',
           '⊄': '\\not\\subset',
           '∘': '\\circ',
           '%': '\%',
           '!': '!',
           '∫∫∫': '\\iiint',
           '∫∫': '\\iint',
           '∫': '\\int',
           '→': '\\rightarrow',
           '←': '\\leftarrow',
           '⤳': '\\rightsquigarrow',
           '⇒': '\\Rightarrow',
           '⇐': '\\Leftarrow',
           '⇔': '\\Leftrightarrow',
           'ℵ': '\\aleph',
           '¬': '\\lnot',
           '∀': '\\forall',
           '∃': '\\exists',
           '∄': '\\nexists',
           '∞': '\\infty',
           '∂': '\\partial',
           'ℝ': '\\mathbb{R}',
           'ℕ': '\\mathbb{N}',
           'ℂ': '\\mathbb{C}',
           'ℤ': '\\mathbb{Z}',
           'ℚ': '\\mathbb{Q}',
           '𝔽': '\\mathbb{F}',
           'ø': '\\emptyset',
           '∅': '\\emptyset',
           '∑': '\\sum',
           '∏': '\\prod',
           '↑': '\\uparrow',
           '♯': '\\sharp',
           '♭': '\\flat',
           '♮': '\\natural',
           '∇': '\\nabla',
           '&': '&',
           '~': '\\sim'}

greeks = {
    'Α': '\\Alpha',
    'Β': '\\Beta',
    'Γ': '\\Gamma',
    'Δ': '\\Delta',
    'Ε': '\\Epsilon',
    'Ζ': '\\Zeta',
    'Η': '\\Eta',
    'Θ': '\\Theta',
    'Ι': '\\Iota',
    'Κ': '\\Kappa',
    'Λ': '\\Lambda',
    'Μ': '\\Mu',
    'Ν': '\\Nu',
    'Ξ': '\\Xi',
    'Ο': '\\Omicron',
    'Π': '\\Pi',
    'Ρ': '\\Rho',
    'Σ': '\\Sigma',
    'Τ': '\\Tau',
    'Υ': '\\Upsilon',
    'Φ': '\\Phi',
    'Χ': '\\Chi',
    'Ψ': '\\Psi',
    'Ω': '\\Omega',
    'α': '\\alpha',
    'β': '\\beta',
    'γ': '\\gamma',
    'δ': '\\delta',
    'ε': '\\varepsilon',
    'ζ': '\\zeta',
    'η': '\\eta',
    'θ': '\\theta',
    'ι': '\\iota',
    'κ': '\\kappa',
    'λ': '\\lambda',
    'μ': '\\mu',
    'ν': '\\nu',
    'ξ': '\\xi',
    'ο': '\\omicron',
    'π': '\\pi',
    'ρ': '\\rho',
    'σ': '\\sigma',
    'ς': '\\varsigma',
    'τ': '\\tau',
    'υ': '\\upsilon',
    'φ': '\\varphi',
    'ɸ': '\\phi',
    'χ': '\\chi',
    'ψ': '\\psi',
    'ω': '\\omega',
    'ℓ': '\\ell'}

# Utility functions
# =================

# This is a decorator that allows us to set the regex associated with
# a token function (i.e. the docstring, __doc__).
def docstring(ds):
    def _decorator(f):
        f.__doc__ = ds
        return f
    return _decorator

# TODO test all of the chars in these tables

# Tokenizer (lex)
# ===============

tokens = ('OPENMATRIX', 'LSBRACKET', 'RSBRACKET', 'NUMBER',
          'LPAREN', 'RPAREN', 'VARIABLE', 'TEXT', 'CARATUNDER',
          'SPACE', 'LBRACKET', 'RBRACKET', 'SEMICOLON', 'COMMA',
          'STRING', 'SQRT', 'SYMBOL', 'LATEX', 'MINUS', 'BAR',
          'DOUBLEBAR', 'LSETBRACKET', 'RSETBRACKET', 'COMBINING',
          'RAWLATEX', 'PRIME')


t_NUMBER = r'[\d\.]+'
t_MINUS = r'-'
t_BAR = r'\|'
t_LPAREN = r'\('
t_RPAREN = r'\)'
t_LSBRACKET = r'\['
t_RSBRACKET = r'\]'
t_CARATUNDER = r'[_^]'
t_SPACE = r'[ \n]+'
t_LBRACKET = r'{'
t_RBRACKET = r'}'
t_SEMICOLON = r';'
t_COMMA = r','
t_STRING = r'\"[A-Za-z ,.!?;]+\"'
t_LATEX = r'\\([A-Za-z0-9]+|\\)'
t_SQRT = r'√'
t_COMBINING = "|".join(combining_chars.keys())
t_RAWLATEX = r'{{.*}}'
t_PRIME = r"'"

precedence = (
    ('left', 'matrowprec'), # For ] as a symbol
    ('left', 'LSBRACKET'), # For ] as a symbol
    ('left', 'primesprec'), # For primes
    ('left', 'PRIME'), # For primes
    ('left', 'simpletermtext'), # Functions beat text + pterm, for functions to a power
    ('left', 'CARATUNDER'), # For functions to a power
    ('left', 'caratminus'), # Below MINUS for x^-1.
    ('left', 'TEXT', 'BAR', 'LATEX', 'VARIABLE', 'SQRT', 'OPENMATRIX'), # For absolute values.  Below termbar for bars in function args
    ('left', 'termbar'), # For absolute values.
    ('left', 'LPAREN'), # Functions beat text + pterm.  Above 'termbar' for absolute values.
    ('left', 'simplelatex'), # Fix x^{\ensuremath{xyz}}.
    ('left', 'LBRACKET'),  # Fix x^{\ensuremath{xyz}}. Above 'termbar' for absolute values.
    ('right', 'MINUS'), # Part of 3 + -3 fix. Above 'termbar' for absolute values.  Above caratminus for x^-1.
    ('left', 'NUMBER'), # Part of 3 + -3 fix. Above 'termbar' for absolute values.
)

def t_DOUBLEBAR(t):
    r'\|\|'
    t.value = r'\|'
    return t

def t_LSETBRACKET(t):
    r'{[ ]+'
    t.value = "\\{ "
    return t

def t_RSETBRACKET(t):
    r'[ ]+}'
    t.value = " \\}"
    return t

def t_TEXT(t):
    r'[A-Za-z]+'
    if t.value == 'mat':
        t.type = 'OPENMATRIX'
    if t.value == 'lim':
        t.type = 'SYMBOL'
        t.value = '\\lim'
    if len(t.value) == 1:
        t.type = 'VARIABLE'
    if len(t.value) == 2 and t.value[0] == 'd': # Hack for differentials
        t.type = 'VARIABLE'
        t.value = '\\text{d}' + t.value[1]
    return t
    # TODO more functions

def t_VARIABLE(t):
    r'[A-Za-zΑ-Ωα-ωɸℓ]'
    global greeks
    if t.value in greeks.keys():
        t.value = greeks[t.value]
    return t

@docstring("|".join(symbols.keys()))
def t_SYMBOL(t):
     # Since + and * must be escaped in regex, we add them in
     # manually.
    fullsymbols = symbols
    fullsymbols.update({'+':'+', '*':'*'})

    if t.value in symbols:
        t.value = symbols[t.value]
    else:
        global parsingstring
        raise MathGrammarError("Symbol value error " + t.value + " in:\n\n" + parsingstring)
    return t

def t_error(t):
    global parsingstring
    raise MathGrammarError("Illegal character '" + t.value[0] + "' in:\n\n" + parsingstring)
    t.lexer.skip(1)


def is_space(s):
    return (re.match(t_SPACE+r'$', s) != None)

def is_newline(s):
    return (re.search('\n', s) != None)

# Parser (yacc)
# =============

# General lists of terms

def p_expressionlist(p):
    """expressionlist : expressionlist COMMA SPACE expression
                      | expressionlist COMMA expression
                      | expression"""
    if len(p) == 5:
        p[0] = p[1] + p[2] + p[3] + p[4]
    elif len(p) == 4:
        p[0] = p[1] + p[2] + p[3]
    elif len(p) == 2:
        p[0] = p[1]

def p_termlist(p):
    """expression : expression SPACE term
                  | expression term"""
    if is_newline(p[2]):
        p[0] = p[1] + "\\\\\n" + p[3]
    elif is_space(p[2]):
        p[0] = p[1] + " " + p[3]
    else:
        p[0] = p[1] + " " + p[2]

def p_termlist_base(p):
    """expression : term"""
    p[0] = p[1]


# Negative numbers

def p_pnnumber(p):
    """pnnumber : MINUS NUMBER
                | NUMBER"""
    if (p[1] == u'-'):
        p[0] = p[1]+p[2]
    else:
        p[0] =  p[1]

def p_pnvariable(p):
    """pnvariable : MINUS VARIABLE
                | VARIABLE"""
    if (p[1] == u'-'):
        p[0] = p[1]+p[2]
    else:
        p[0] =  p[1]

# Common types of variables

def p_term_dfn(p):
    """term : TEXT %prec simpletermtext
            | mathterm
            | MINUS
            | BAR %prec termbar
            | DOUBLEBAR
            | RSETBRACKET
            | LSETBRACKET
            | coordpair
            | SYMBOL"""
    p[0] = p[1]
    # Substitute symbols

def p_mathterm(p):
    """mathterm : pnvariable
                | pnnumber
                | LATEX %prec simplelatex
                | latextended
                | function
                | sqrtterm
                | matrix
                | pterm"""
    p[0] = p[1]

def p_rawlatex(p):
    'mathterm : RAWLATEX'
    p[0] = p[1][2:len(p[1])-2]

def p_term_string(p):
    'term : STRING'
    p[0] = "\\text{"+p[1].strip("\"")+"}"

def p_term_variable_caratunder_term(p):
    """term : term CARATUNDER mathterm
            | term CARATUNDER coordpair
            | term CARATUNDER SYMBOL
            | term CARATUNDER TEXT
            | term CARATUNDER MINUS %prec caratminus"""
    p[0] = p[1] + p[2] + "{" + p[3] + "}"

def p_term_variable_caratunder_string(p):
    """term : term CARATUNDER STRING"""
    noquotes = p[3].strip("\"")
    p[0] = p[1] + p[2] + "\\text{" + noquotes + "}"

# This handles xy^something, knowing that it isn't a function.
def p_exp_text(p):
    """term : TEXT CARATUNDER SYMBOL
            | TEXT CARATUNDER MINUS %prec caratminus
            | TEXT CARATUNDER coordpair
            | TEXT CARATUNDER mathterm
            | TEXT CARATUNDER TEXT
            | TEXT primes"""
    if len(p) == 4:
        p[0] = p[1] + p[2] + "{" + p[3] + "}"
    else:
        p[0] = p[1] + p[2]

def p_exp_string(p):
    """term : TEXT CARATUNDER STRING"""
    noquotes = p[3].strip("\"")
    p[0] = p[1] + p[2] + "\\text{" + noquotes + "}"



def p_mathterm_prime(p):
    """term : term primes %prec primesprec"""
    p[0] = p[1] + p[2]

def p_primes(p):
    """primes : primes PRIME
              | PRIME"""
    if len(p) == 3:
        p[0] = p[1] + p[2]
    else:
        p[0] = p[1]

def p_combine(p):
    """mathterm : mathterm COMBINING"""
    if p[2] in combining_chars:
        p[0] = '\\'+combining_chars[p[2]]+'{'+p[1]+'}'
    else:
        p[0] = p[1]
        global parsingstring
        raise MathGrammarError("Combining term error " + p[2] + " in:\n\n" + parsingstring)
    #TODO Add "\circled" command to latex header

def p_coordinate_pair(p):
    """coordpair : LBRACKET arglist RBRACKET
                 | LPAREN arglist RPAREN """
    p[0] = p[1] + p[2] + p[3]

def p_interval(p):
    """mathterm : LSBRACKET arglist RSBRACKET
                | LPAREN arglist RSBRACKET
                | LSBRACKET arglist RPAREN"""
    p[0] = p[1] + p[2] + p[3]    

# Parentheticals

def p_term_lparen_expression_rparen(p):
    'pterm : LPAREN expression RPAREN'
    p[0] = p[1] + p[2] + p[3]

def p_term_lbracket_expression_rbracket(p):
    'mathterm : LBRACKET expression RBRACKET'
    p[0] = p[2]

# Latex

def p_latextended(p):
    'latextended : LATEX latextendedargs'
    p[0] = p[1] + p[2]

def p_latextended_args(p):
    """latextendedargs : latextendedargs LBRACKET expression RBRACKET
                       | LBRACKET expression RBRACKET"""
    if len(p) == 5:
        p[0] = p[1] + p[2] + p[3] + p[4]
    else:
        p[0] = p[1] + p[2] + p[3]

# Functions

def p_function(p):
    """function : TEXT pterm
                | VARIABLE pterm
                | TEXT CARATUNDER pnnumber pterm
                | VARIABLE CARATUNDER pnnumber pterm
                | TEXT coordpair
                | TEXT primes coordpair
                | VARIABLE primes coordpair
                | TEXT primes pterm
                | VARIABLE primes pterm
                | TEXT CARATUNDER pnnumber coordpair
                | VARIABLE CARATUNDER pnnumber coordpair
                | VARIABLE coordpair"""
    if p[2] in ['^', '_']:
        decor = p[2] + '{' + p[3] + '}'
        args = p[4]
    else:
        decor = ""
        args = p[2]
    if p[1] in funs + ['E']: # Expected value in roman since it is just one letter
        p[0] = "\\text{" + p[1] + "}" + decor + args
        # When exported to latex, a normal latex-style function (like
        # \sin) will typeset in computer modern rather than than
        # chosen font.  This is a bug in pandoc which could be fixed
        # by adding "\setmathrm{FONTNAME}" to the template.  To fix
        # this, and to maintain compatibility with new spontaneous
        # functions, we make it ourselves.
    else:
        p[0] = p[1] + decor + args
    # TODO functions with super/subscripts

def p_arglist_arglist_comma_expression(p):
    """arglist : expression COMMA expression
               | arglist COMMA expression
               | expression SEMICOLON expression
               | arglist SEMICOLON expression
               | expression COMMA SPACE expression
               | arglist COMMA SPACE expression
               | expression SEMICOLON SPACE expression
               | arglist SEMICOLON SPACE expression"""
    if is_space(p[3]):
        p[0] = p[1] + p[2] + " " + p[4]
    else:
        p[0] = p[1] + p[2] + " " + p[3]


def p_expression_sqrt_num(p):
    """sqrtterm : SQRT mathterm"""
    p[0] = "\\sqrt{"+p[2]+"}"

# Matrices.  Assuming commas for now.

def p_matrix(p):
    """matrix : OPENMATRIX LSBRACKET matrixrowsc RSBRACKET"""
    p[0] = "\\begin{bmatrix}\n" + p[3] + "\\end{bmatrix}"

def p_matrixrowsc(p):
    """matrixrowsc : matrixrowsc SEMICOLON matrixrowc
                   | matrixrowsc SEMICOLON SPACE matrixrowc
                   | matrixrowc"""
    if len(p) == 2:
        p[0] = p[1]
    elif is_space(p[3]):
        p[0] = p[1] + "\\\\\n" + p[4]
    else:
        p[0] = p[1] + "\\\\\n" + p[3]

def p_matrixrowc(p):
    """ matrixrowc : matrixrowc COMMA SPACE expression %prec matrowprec
                   | matrixrowc COMMA expression %prec matrowprec
                   | expression %prec matrowprec"""
    if len(p) == 2:
        p[0] = p[1]
    elif is_space(p[3]):
        p[0] = p[1] + "&" + p[4]
    else:
        p[0] = p[1] + "&" + p[3]

# The above are suffixed with a "c" in case spaces as delimiters for
# matrices are ever added in the future.  It appears that the parser
# is not powerful enough to do this (notably, to recognize that the
# matrix mat[x y, z] is a comma-delimited matrix, not a space one) so
# I will leave it out for right now.

def p_error(p):
    global parsingstring
    if p == None: # Ply's notation for EOF
        raise MathGrammarError("EOF encountered unexpectedly in:\n\n"+parsingstring+"\n\nDo you have an extra open parenthesis or bracket, or did \nyou forget to leave space between two adjacent variables?")
    else:
        raise MathGrammarError("Unexpected " + p.value + " in: \n\n"+parsingstring)


lexer = lex.lex(reflags=re.UNICODE)
parser = yacc.yacc()
parsingstring = "" # Set to whatever string was last parsed by parse_math (for verbose errors)

# Function to convert simplified math to latex-compatible math
def parse_math(s):
    global parsingstring
    parsingstring = s # For displaying errors
    return parser.parse(s)

if __name__ == "__main__":
    s = raw_input()
    parsingstring = s
    print parse_math(s)

# Note to future self: When debugging shift/reduce problems, remember
# how the precedence rules work.  When yacc is deciding whether to
# shift or reduce, it looks at the next token coming up (the token in
# the precedence rules) and the potential symbol it can reduce to (the
# %prec tag in the precedence rules).  It shifts if the token has a
# higher precedence than the potential symbol.  So to force it to
# shift, set the token with a higher precedence.
