#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

# UTF8?
import codecs, sys
sys.stdout = codecs.getwriter('utf-8')(sys.stdout)

# More UTF8?
import sys
reload(sys)
sys.setdefaultencoding('utf-8')

"""Pandoc filter for allowing angle bracket links to files, such as
<notes3-mode.el>.  This also integrates with datadirfilter.py.
Presently, this is not compatible with filenames that contain spaces.
As a convenience feature, it also allows angle bracket links to
images.  Note that this filter requires raw_html to be disabled, or
your results may vary.

"""

from pandocfilters import Link, Image, Str, walk
import sys # for stderr debugging
import os # For checking if files exist
import json # Instead of toJSONFilter
import re # For testing for brackets



# Take a JSON object as input.  The following is the structure of the
# link and image objects:
# 
# {t: "Str"
#  c: The value of the string }
#
# {t: "RawInline"
#  c: ["html" "The value of the string" }
#
# {t: "Link" or "Image"
#  c: [ [ Inline elements ]
#       [ "Link url", "Link title (optional)" ]
#     ] }
# 
# This will find everything that looks like a link and see if it
# refers to a local file.  If so, link to that file.  
def shortfilelinksproc(key, value, format, meta):
    if key == 'Str':
        m = re.match(r'^!?<(.*)>(.*)$', value)
        if m == None: return
        path = m.group(1)
        if check_if_file_exists(path) != None:
            # There is no way to check to see if another filter is in
            # use, and whether or not it is before or after the
            # present filter in the filter chain.  We err here on the
            # side of caution, assuming that it is.  However, this
            # introduces a small bug that will likely never surface:
            # if someone links to file.txt in the note note.n (for
            # instance) and both note.n.d/file.txt and
            # note.n.d/note.n.d/file.txt exist, then the latter will
            # be preferred if this filter runs before
            # datadirfilter.py.
            linkproperties = [check_if_file_exists(path), path]
            if m.group()[0] == "!":
                if m.group(2):
                    return [Image(("", [], []), [Str(path)], linkproperties), Str(m.group(2))]
                else:
                    return [Image(("", [], []), [Str(path)], linkproperties)]
            else:
                if m.group(2):
                    return [Link(("", [], []), [Str(path)], linkproperties), Str(m.group(2))]
                else:
                    return Link(("", [], []), [Str(path)], linkproperties)

# Here is an alternative version of the function that is *almost*
# compatible with having the raw_html pandoc extension enabled.  The
# only problem with it is it leaves an exclamation point in front of
# some images.  It is possible to fix this problem in this code, but
# (it seems) impossible to do it elegantly.

# lastwasexclam = False # Explained below
# def shortfilelinksproc(key, value, format, meta):
#     # This allows us to still use the "walk" method.  When there are
#     # no slashes in the path, pandoc detects "!<path>" as two separate
#     # elements: "!" (a Str) and "<path>" (a RawInline).  So, we have a
#     # flag describing whether the last element was a Str with the
#     # contents "!".  This hack requires that the "walk" method starts
#     # at the top of the document and goes down sequentially, element
#     # by element.
#     global lastwasexclam
#     prevwasexclam = lastwasexclam
#     lastwasexclam = False
#     # Now continue...
#     if key == 'Str':
#         lastwasexclam = True if value == "!" else False
#         if re.match(r'^!?<.*>$', value):
#             path = value.strip('<>!')
#             if check_if_file_exists(path) != None:
#                 # There is no way to check to see if another filter is
#                 # in use.  We err here on the side of caution,
#                 # assuming that it is.  However, this introduces a
#                 # small bug that will likely never surface: if someone
#                 # links to file.txt in the note note.n (for instance)
#                 # and both note.n.d/file.txt and
#                 # note.n.d/note.n.d/file.txt exist, then the latter
#                 # will be preferred if this filter runs before
#                 # datadirfilter.py.
#                 linkproperties = [check_if_file_exists(path), path]
#                 if value[0] == "!":
#                     return Image([Str(path)], linkproperties)
#                 else:
#                     return Link([Str(path)], linkproperties)
#     elif key == 'RawInline':
#         [t, v] = value
#         if re.match('^<.*>$', v): # No "!" because that doesn't detect as a RawInline
#             v = v.strip('<>')
#             if check_if_file_exists(v) != None:
#                 linkproperties = [check_if_file_exists(v), v]
#                 if prevwasexclam:
#                     return Image([Str(v)], linkproperties)
#                 else:
#                     return Link([Str(v)], linkproperties)

if __name__ == "__main__":
    doc = json.loads(sys.stdin.read())
    if len(sys.argv) > 1:
        format = sys.argv[1]
    else:
        format = ""
    meta = doc[0]['unMeta']
    # If datadirfilter.py is present in the same directory, import it.
    # It would be preferable to leave the heavy lifting to this filter
    # to keep the features completely separate, but alas, we only want
    # to link to files in angle brackets if the file actually exists,
    # and we can't know this without looking in the .d directory.
    try:
        from datadirfilter import datadir_path
        datadir = datadir_path(meta)
        assert(datadir != None)
        # Returns the path to the file if the file does exist, or None
        # if it does not.  Note this is a conditional definition if
        # the import is successful, if not, use the one below.
        def check_if_file_exists(f):
            if os.path.exists(datadir + f):
                return datadir + f
            elif os.path.exists(f):
                return f
            return None
    except (ImportError, AssertionError):
        # Alternative definition for check_if_file_exists.
        def check_if_file_exists(f):
            if os.path.exists(f):
                return f
            return None
    altered = walk(doc, shortfilelinksproc, format, meta)
    json.dump(altered, sys.stdout)


# FIXME One of the downsides to removing inline HTML is losing HTML comments.
